#ifndef	ARRAYOB_H
#define	ARRAYOB_H

/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/ArrayOb.h,v 3.12 92/07/26 14:31:48 sandy Exp $*/

/* ArrayOb.h -- declarations for array of object pointers

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

$Log:	ArrayOb.h,v $
 * Revision 3.12  92/07/26  14:31:48  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.11  92/06/17  11:36:56  sandy
 * __DECCXX compatible
 * 
 * Revision 3.10  91/02/18  21:05:33  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.0  90/05/20  00:18:56  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Collection.h>

class ArrayOb: public Collection {
	DECLARE_MEMBERS(ArrayOb);
	Object** v;
	unsigned sz;
	void allocSizeErr() const;
	void indexRangeErr() const;
protected:		// storer() functions for object I/O
	virtual void storer(OIOofd&) const;
	virtual void storer(OIOout&) const;
public:
	ArrayOb(unsigned size =DEFAULT_CAPACITY);
	ArrayOb(const ArrayOb&);
	~ArrayOb();
	Object*& elem(int i)			{ return v[i]; }
#ifdef __DECCXX
// Workaround _DECCXX bug failing to disambiguate elem() and elem() const
	const Object *const& elem(long i) const	{ return v[i]; }
#else
	const Object *const& elem(int i) const	{ return v[i]; }
#endif
	bool operator!=(const ArrayOb& a) const	{ return !(*this==a); }
	void operator=(const ArrayOb&);
	bool operator==(const ArrayOb&) const;
	Object*& operator[](int	i)	{
		if ((unsigned)i >= sz) indexRangeErr();
		return v[i];
	}
#ifdef __DECCXX
// Workaround _DECCXX bug failing to disambiguate operator[]() and operator[]() const
	const Object *const& operator[](long i) const	{
		if ((unsigned)i >= sz) indexRangeErr();
		return v[i];
	}
#else
	const Object *const& operator[](int i) const	{
		if ((unsigned)i >= sz) indexRangeErr();
		return v[i];
	}
#endif
	virtual Collection& addContentsTo(Collection&) const;
	virtual Object*& at(int i);
#ifdef __DECCXX
// Workaround _DECCXX bug failing to disambiguate at() and at() const
	virtual const Object *const& at(long i) const;
#else
	virtual const Object *const& at(int i) const;
#endif
	virtual unsigned capacity() const;
	virtual int compare(const Object&) const;
	virtual void deepenShallowCopy();
	virtual Object* doNext(Iterator&) const;
	virtual unsigned hash() const;
	virtual bool isEqual(const Object&) const;
	virtual void reSize(unsigned);
	virtual void removeAll();
	virtual unsigned size() const;
	virtual void sort();
	virtual const Class* species() const;
private:			    // shouldNotImplement();
	virtual	Object* add(Object&);
	virtual unsigned occurrencesOf(const Object&) const;
	virtual Object* remove(const Object&);
};

#endif
