/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Array_h.m4,v 3.16 92/09/11 13:49:16 sandy Exp $*/

/* Array_h.m4 -- template for declarations for generic array objects

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	September, 1985

Function:

m4 macro template for the .h files for arrays of the specified
fundamental type: char, int, short, long, unsigned, float, and double.
For example, to generate the declarations for an array of chars:

	echo "ARRAYDECLARE(Arraychar,char,friend SubString;)" | m4 Array.h.m4 - >Arraychar.h

$Log:	Array_h.m4,v $
Revision 3.16  92/09/11  13:49:16  sandy
g++ compatible

Revision 3.15  92/07/26  14:30:23  sandy
include directive in form '#include <nihcl/foo.h>'

Revision 3.14  92/06/17  11:43:53  sandy
__DECCXX compatible

Revision 3.13  92/05/28  16:37:44  sandy
corrected overloaded functions

Revision 3.12  92/05/28  16:36:34  sandy
corrected overloaded functions

Revision 3.11  92/05/25  11:51:20  sandy
__DECCXX compatible

Revision 3.10  91/02/18  21:05:35  kgorlen
Release for 3rd printing of 1st edition.

Revision 3.1  90/11/10  18:28:38  kgorlen
Change return type of operator[]() from value to
const reference.

Revision 3.0  90/05/20  00:18:58  kgorlen
Release for 1st edition.

*/

define(ARRAYDECLARE,
``#ifndef'' $1_H
``#define'' $1_H

``#include'' <nihcl/Collection.h>
``#if'' defined(__DECCXX) || defined(__GNUG__)
``#include'' <stdlib.h>
``#else''
``#include'' <malloc.h>
``#endif''

inline $2* REALLOC($2* ptr, unsigned size)
{
	return ($2*)realloc((char*)ptr,sizeof($2)*size);
}

inline void DELETE($2* ptr) { free((char*)ptr); }

class $1: public Collection {
	DECLARE_MEMBERS($1);
	$2* v;
	unsigned sz;
	void AllocSizeErr() const;
	void IndexRangeErr() const;
// friends go here
	$3
protected:		// storer() functions for object I/O
	virtual void storer(OIOofd&) const;
	virtual void storer(OIOout&) const;
public:
	$1(unsigned size =DEFAULT_CAPACITY);
	$1(const $1&);
	~$1();
	$2& elem(int i)			{ return v[i]; }
``#ifdef __DECCXX''
// Workaround __DECCXX bug failing to disambiguate elem() and elem() const
	const $2& elem(long i) const	{ return v[i]; }
``#else''
	const $2& elem(int i) const	{ return v[i]; }
``#endif''
	bool operator!=(const $1& a) const { return !(*this==a); }
	void operator=(const $1&);
	bool operator==(const $1&) const;
	$2& operator[](int i) {
		if ((unsigned)i >= sz) IndexRangeErr();
		return v[i];
	}
``#ifdef __DECCXX''
// Workaround __DECCXX bug failing to disambiguate elem() and elem() const
	const $2& operator[](long i) const {
``#else''
	const $2& operator[](int i) const {
``#endif''
		if ((unsigned)i >= sz) IndexRangeErr();
		return v[i];
	}
	virtual unsigned capacity() const;
	virtual int compare(const Object&) const;
	virtual void deepenShallowCopy();
	virtual void dumpOn(ostream& strm =cerr) const;
	virtual unsigned hash() const;
	virtual bool isEqual(const Object&) const;
	virtual void printOn(ostream& strm =cout) const;
	virtual void removeAll();
	virtual void reSize(unsigned);
	virtual unsigned size() const;
	virtual void sort();
	virtual const Class* species() const;
private:					// shouldNotImplement()
	virtual Object* add(Object&);
	virtual Object*& at(int);
``#ifdef __DECCXX''
// Workaround __DECCXX bug failing to disambiguate elem() and elem() const
	virtual const Object *const& at(long) const;
``#else''
	virtual const Object *const& at(int) const;
``#endif''
	virtual Object* doNext(Iterator&) const;
	virtual unsigned occurrencesOf(const Object&) const;
	virtual Object* remove(const Object&);
};

``#endif''
)
