/* Array_c.m4 -- template for implementations of generic array objects

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	September, 1985

Function:

m4 macro template for the .c files for arrays of the specified
fundamental type: char, int, short, long, unsigned, float, and double.
For example, to generate the implementation of an array of chars:

	m4 Array.c.m4 Arraychar.p >Arraychar.c

The type-specific part of the implementation is in the file
Arraychar.p, for example.

WARNING -- Make changes to the .p or .m4 files, not to the .c file
they generate.

$Log:	Arraychar.c,v $
 * Revision 1.2  92/07/26  14:31:48  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 1.1  92/04/30  15:47:12  sandy
 * Initial revision
 * 
Revision 3.10  91/02/18  21:05:34  kgorlen
Release for 3rd printing of 1st edition.

Revision 3.0  90/05/20  00:18:57  kgorlen
Release for 1st edition.

*/

#define NEW(type,size)	((type*)malloc(sizeof(type)*(size)))


/* Arraychar.p -- type-specific functions for class Arraychar

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	September, 1985

Function:
	
Type-specific functions for class Arraychar, such as hash() and
printOn().  Generic functions are generated from the m4 template file
Array.c.m4.

WARNING -- Make changes to the .p or .m4 files, not to the .c file
they generate.

$Log:	Arraychar.c,v $
 * Revision 1.2  92/07/26  14:31:48  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 1.1  92/04/30  15:47:12  sandy
 * Initial revision
 * 
 * Revision 3.11  91/07/16  13:12:14  kgorlen
 * Simplify I/O statement to compile with iomanip.h templates.
 * 
 * Revision 3.10  91/02/18  21:05:36  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.0  90/05/20  00:19:00  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Arraychar.h>
#include <nihcl/nihclconfig.h>
#include <iomanip.h>
#include <libc.h>
#include <nihcl/nihclIO.h>

#define	THIS	Arraychar
#define	BASE	Collection
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_CLASS(Arraychar,1,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Arraychar.c,v 1.2 92/07/26 14:31:48 sandy Exp Locker: sandy $",NULL);

static int charCmp(const void* a, const void* b)
{
	return *(const unsigned char*)a - *(const unsigned char*)b;
}

union hash_char_mask {
	unsigned in[sizeof(int)];
	char ch[sizeof(int)*sizeof(int)];
	hash_char_mask();
} mask;

hash_char_mask::hash_char_mask()
{
	for (register unsigned i=0; i<sizeof(int); i++) {
		for (register unsigned j=0; j<sizeof(int); j++) ch[sizeof(int)*i+j] = j<i ? 0xff : 0;
	}
}

unsigned THIS::hash() const
{
	register unsigned h = sz;
	register unsigned i = div_sizeof_int(sz);
	register unsigned* vv = (unsigned*)v;
	while (i--) h ^= *vv++;
	if ((i = mod_sizeof_int(sz)) != 0)
		h ^= *vv & mask.in[i];
	return h;
}

void THIS::printOn(ostream& strm) const
{
	for (register unsigned i=0; i<sz; i++) {
		if (i > 0) strm << "  ";
 		strm << "0x";
		strm << setfill('0');
		strm << setw(2) << hex << ((unsigned int)(unsigned char)v[i]);
	}
	strm << dec;
}

THIS::THIS(OIOin& strm)
:
#ifdef MI
	Object(strm),
#endif
	BASE(strm)
{
	strm >> sz;
	v = NEW(char,sz);
	strm.get(v,sz);
}

void THIS::storer(OIOout& strm) const
{
	BASE::storer(strm);
	strm << sz;
	strm.put(v,sz);
}

Arraychar::Arraychar(OIOifd& fd)
:
#ifdef MI
	Object(fd),
#endif
	BASE(fd)
{
	fd >> sz;
	v = NEW(char,sz);
	fd.get(v,sz);
}

void Arraychar::storer(OIOofd& fd) const
{
	BASE::storer(fd);
	fd << sz;
	fd.put(v,sz);
}

Arraychar::Arraychar(unsigned size)
{
	sz = size;
	if (size==0) AllocSizeErr();
	v = NEW(char,sz);
	removeAll();
}

Arraychar::Arraychar(const Arraychar& a)
{
	register unsigned i = a.sz;
	sz = i;
	v = NEW(char,i);
	register char* vv = v;
	register char* av = a.v;
	while (i--) *vv++ = *av++;
}

Arraychar::~Arraychar()	{ DELETE(v); }

void Arraychar::operator=(const Arraychar& a)
{
	if (v != a.v) {
		DELETE(v);
		v = NEW(char,sz=a.sz);
		register unsigned i = a.sz;
		register char* vv = v;
		register char* av = a.v;
		while (i--) *vv++ = *av++;
	}
}

bool Arraychar::operator==(const Arraychar& a) const
{
	if (sz != a.sz) return NO;
	register unsigned i = sz;
	register char* vv = v;
	register char* av = a.v;
	while (i--) if (*vv++ != *av++) return NO;
	return YES;
}

unsigned Arraychar::capacity()	const { return sz; }

int Arraychar::compare(const Object& p) const
{
	assertArgSpecies(p,classDesc,"compare");
	const Arraychar& a = castdown(p);
	register unsigned i = MIN(sz,a.sz);
	register char* vv = v;
	register char* av = a.v;
	while (i--) {
		if (*vv != *av) {
			if (*vv > *av) return 1;
			return -1;
		}
	}
	if (sz > a.sz) return 1;
	if (sz == a.sz) return 0;
	return -1;
}

void Arraychar::dumpOn(ostream& strm) const
{
	strm << className() << '[';
	printOn(strm);
	strm << "]\n";
}

bool Arraychar::isEqual(const Object& a) const
{
	return a.isSpecies(classDesc) && *this==castdown(a);
}

void Arraychar::removeAll()
{
	register unsigned i = sz;
	register char* vv = v;
	while (i--) *vv++ = 0;
}

const Class* Arraychar::species() const { return Arraychar::desc(); }

void Arraychar::reSize(unsigned newsz)
{
	if (newsz<=0) AllocSizeErr();
	v = REALLOC(v,newsz);
	sz = newsz;
}

void Arraychar::deepenShallowCopy()
{
	BASE::deepenShallowCopy();
}

unsigned Arraychar::size() const	{ return sz; }

void Arraychar::sort()
{
	qsort(v,sz,sizeof(char),charCmp);
}

extern const int NIHCL_ALLOCSIZE;
extern const int NIHCL_INDEXRANGE;

void Arraychar::AllocSizeErr() const
{
	setError(NIHCL_ALLOCSIZE,DEFAULT,this,className());
}

void Arraychar::IndexRangeErr() const
{
	setError(NIHCL_INDEXRANGE,DEFAULT,this,className());
}

Object* Arraychar::add(Object&)
{
	shouldNotImplement("add");
	return 0;
}

Object*& Arraychar::at(int)
{
	shouldNotImplement("at");
	return (Object*&)nil;
}

const Object *const& Arraychar::at(int) const
{
	shouldNotImplement("at");
	return (Object *const&)nil;
}

Object* Arraychar::doNext(Iterator&) const
{
	shouldNotImplement("doNext");
	return 0;
}

unsigned Arraychar::occurrencesOf(const Object&) const
{
	shouldNotImplement("occurrencesOf");
	return 0;
}

Object* Arraychar::remove(const Object&)
{
	shouldNotImplement("remove");
	return 0;
}

