/* Class.c -- implementation of class descriptor objects

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	September, 1985

Function:
	
Functions pertaining to class Class.  Most of these are defined inline
in object.h.

Modification History:

$Log:	Class.c,v $
 * Revision 3.16  92/12/19  15:36:41  sandy
 * added some TRACE output
 * 
 * Revision 3.15  92/07/26  14:31:58  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.14  92/06/17  11:38:19  sandy
 * *** empty log message ***
 * 
 * Revision 3.13  92/05/25  18:44:30  sandy
 * Workaround __DECCXX problem with casting Foo** to const Foo**
 * 
 * Revision 3.12  92/04/30  15:36:44  sandy
 * new member Class::dictionary()
 * static class StaticMarker to mark arrival of class descriptor constructions
 * use one initor instead of initor1 and initor2
 * Class::initialize obsolete
 * 
 * Revision 3.11  91/07/16  13:19:02  kgorlen
 * Modify Class:readFrom() functions to set pointer to
 * descriptor of class being read by calling
 * ReadFromTbl::readFromClass.
 * 
 * Revision 3.10  91/02/18  21:06:01  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.5  91/02/05  13:07:45  kgorlen
 * Eliminate reference to NIHCL_UNKCLASS.
 * 
 * Revision 3.4  90/12/04  23:08:33  kgorlen
 * Condition stdarg fix on FIX_STDARG.
 * 
 * Revision 3.3  90/11/25  00:00:33  kgorlen
 * Add calls to ReadFromTbl::addObjectEnable() to
 * Class::readObject().
 * 
 * Revision 3.2  90/11/13  22:11:39  kgorlen
 * Remove Object I/O related members from Class;
 * inherit from ReadFromTbl and StoreOnTbl instead.
 * 
 * Revision 3.1  90/11/10  18:29:32  kgorlen
 * Conditionalize sparc vararg hack on SUNCC.
 * Eliminate Class(OIOifd&) and Class(OIOin&).
 * 
 * Revision 3.0  90/05/20  00:19:16  kgorlen
 * Release for 1st edition.
 * 
*/

#include <stdarg.h>
#include <string.h>

#include <nihcl/Object.h>
#include <nihcl/String.h>
#include <nihcl/Dictionary.h>
#include <nihcl/Assoc.h>
#include <nihcl/OrderedCltn.h>
#include <nihcl/IdentSet.h>
#include <nihcl/nihclIO.h>
#include <nihcl/OIOTbl.h>

extern const int NIHCL_RDBADSIG,NIHCL_RDBADTYP,NIHCL_RDWRONGCLASS,NIHCL_RDUNKCLASS;

#define	THIS	Class
#define	BASE	Object
#define	BASE_CLASSES	BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES Object::desc()

/* DEFINE_CLASS */

Object* Class::reader(OIOin&)
{
	setError(NIHCL_RDUNKCLASS,DEFAULT,"Class");
	return 0;
}
Object* Class::reader(OIOifd&)
{
	setError(NIHCL_RDUNKCLASS,DEFAULT,"Class");
	return 0;
}

static Class class_Class("Class",
	ClassList(0,BASE_CLASSES,0), ClassList(0,0), ClassList(0,0),
	1,	// version
	"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Class.c,v 3.16 92/12/19 15:36:41 sandy Exp Locker: sandy $",
	sizeof(Class), Class::reader, Class::reader, NULL );

const Class* Class::desc()	{ return &class_Class; }

const Class* Class::isA() const	{ return &class_Class; }

Object* Class::shallowCopy() const	{ return new Class(*this); }

_DEFINE_CASTDOWN(Class)

// head of the list of all Classes
// built by the static constructor of class Class 
static Class* allClasses =NULL;

// Dictionary of all Classes 
static Dictionary* classDictionary =NULL;
const Dictionary& Class::dictionary()
{
    if ( classDictionary==NULL ) {
        classDictionary = new Dictionary;
        for ( Class* clp = allClasses; clp; clp=clp->nextClass )
#ifdef TRACE
{
cout << "dictionary(): adding " << clp->name() << " to class Dictionary" << endl;
#endif
	    classDictionary->add(*new Assoc(*new String(clp->name()), *clp));
#ifdef TRACE
}
#endif
    }
    return *classDictionary;
}

/*
Dictionary& Class::dictionary = classDictionary;
// mark classes needed for a Dictionary
static class StaticMarker {
	int mark_Object, mark_Collection, mark_Set, mark_Dictionary;
	int mark_String, mark_LookupKey, mark_Assoc, mark_Class;
public:
	StaticMarker() {}
	void mark(Class*);
	int ismarked();
} staticmarker;

// StaticMarker marks arrival of class descriptors
// for those classes needed to use a Dictionary
void StaticMarker::mark(Class* cl)
{
    if ( strcmp(cl->name(),"Object")==0 )      { mark_Object=1; return; }
    if ( strcmp(cl->name(),"Collection")==0 )  { mark_Collection=1; return; }
    if ( strcmp(cl->name(),"Set")==0 )         { mark_Set=1; return; }
    if ( strcmp(cl->name(),"Dictionary")==0 )  { mark_Dictionary=1; return; }
    if ( strcmp(cl->name(),"String")==0 )      { mark_String=1; return; }
    if ( strcmp(cl->name(),"LookupKey")==0 )   { mark_LookupKey=1; return; }
    if ( strcmp(cl->name(),"Assoc")==0 )       { mark_Assoc=1; return; }
    if ( strcmp(cl->name(),"Class")==0 )       { mark_Class=1; return; }
}
int StaticMarker::ismarked()
{
    return (mark_Object==1&&mark_Collection==1&&mark_Set==1&&
            mark_Dictionary==1&&mark_String==1&&mark_LookupKey==1&&
	    mark_Assoc==1&&mark_Class==1);
}
// flag initialization of classDictionary
int is_init_classDictionary = 0;
*/

#if defined(sparc) && defined(FIX_STDARG)
// Use magic name to make stdarg work
#define arg0 __builtin_va_alist
#endif

ClassList::ClassList(const char* arg0, ...)
{
	va_list ap;
	unsigned nargs = 0;
	va_start(ap, arg0);
	do nargs++; while (va_arg(ap, Class*));
	va_end(ap);
	Class** p = clp = new Class*[nargs];
	va_start(ap, arg0);
	while (nargs--) *p++ = va_arg(ap, Class*);
	va_end(ap);
}

Class::Class(const char* name,
	const ClassList& bases, const ClassList& members, const ClassList& vbases,
	unsigned version, const char* ident, unsigned size,
	Object* (*reader)(OIOin&),
	Object* (*binreader)(OIOifd&),
	initorTy initor )
:
	class_name(name),
	class_ident(ident),
#ifndef __DECCXX
	class_bases(bases.clp),
	class_members(members.clp),
	class_vbases(vbases.clp),
#else
// Workaround __DECCXX problem with casting Foo** to const Foo**
	class_bases((const Class**)bases.clp),
	class_members((const Class**)members.clp),
	class_vbases((const Class**)vbases.clp),
#endif
	class_version(version),
	inst_size(size),
	inst_reader(reader),
        inst_binreader(binreader)
{
	nextClass = allClasses; allClasses = this;
#ifdef TRACE
cout << "prepending " << name << " to allClasses" << endl;
#endif
	class_signature = 0;
	class_number = 0;
/*
	if ( is_init_classDictionary )
// use classDictionary if already initialized by its static constructor
	    dictionary.add(*new Assoc(*new String(class_name), *this));
	else {
// record class in staticmarker
	    staticmarker.mark(this);
	    if ( staticmarker.ismarked() ) {
// when all marking is completed load classDictionary with allClasses
		is_init_classDictionary = 1;
	        for ( Class* clp = allClasses; clp; clp=clp->nextClass )
		    dictionary.add(*new Assoc(*new String(clp->name()), *clp));
		}
	}
*/
//	if (initor != 0) initor(*this);	MASSCOMP cc bug
	initorTy initfun = initor;
	if (initfun != 0) initfun(*this);
}

Class::Class(const Class& c)
:
	class_name(c.class_name),
	class_bases(c.class_bases),
	class_members(c.class_members),
	class_vbases(c.class_vbases),
	class_version(c.class_version),
	class_ident(c.class_ident),
	class_signature(c.class_signature),
	inst_size(c.inst_size),
	inst_reader(c.inst_reader),
        inst_binreader(c.inst_binreader),
	class_number(c.class_number)
{
}

int Class::compare(const Object& ob) const	// compare Class names 
{
	assertArgSpecies(ob,class_Class,"compare");
	return strcmp(name(),castdown(ob).name());
}

unsigned Class::hash() const	{ return (const unsigned)this; }

bool Class::isEqual(const Object& ob) const	{ return this == castdown(&ob); }

bool Class::_isKindOf(const Class& clid) const
{
    if (this == &clid) return YES;
    const Class** bp = baseClasses();
    while (*bp) if ((*bp++)->_isKindOf(clid)) return YES;
    return NO;
}

void Class::deepenShallowCopy()	{ shouldNotImplement("deepenShallowCopy"); }

void Class::dumpOn(ostream& strm) const
{
	strm << className() << '[' << name() << '\n';
	strm << "ident: " << ident() << '\n';
	strm << "version: " << version() << '\n';
	strm << "signature: " << signature() << '\n';
	strm << "number: " << number() << '\n';
	strm << "base classes:";
	const Class** p = baseClasses();
	while (*p) strm << ' ' << (*p++)->name();
	strm << '\n';
	strm << "member classes:";
	p = memberClasses();
	while (*p) strm << ' ' << (*p++)->name();
	strm << '\n';
	strm << "virtual base classes:";
	p = virtualBaseClasses();
	while (*p) strm << ' ' << (*p++)->name();
	strm << '\n';
	strm << "]\n";
}

void Class::printOn(ostream& strm) const	{ strm << class_name; }

unsigned Class::size() const	{ return inst_size; }
	
void Class::addSignature(unsigned long s)
{
// Assumes that an unsigned long is at least 32 bits
	class_signature = ((class_signature << 3) + (class_signature >> (32-3) & 7) + s) & 0xFFFFFFFF;
}

// computeSignature assumes signatures never computed from static constructors
// so that all class descriptor are already initialized
void Class::computeSignature()
{
	if (this == Object::desc()) {
		class_signature = class_version;
		return;
	}

// compute signature
	Class** p = (Class**)class_bases;
	while (*p) addSignature((*p++)->signature());
	if (class_signature == 0) class_signature++;
	p = (Class**)class_members;
	while (*p) addSignature((*p++)->signature());
	if (class_version != 0) addSignature(class_version);
	if (class_signature == 0) class_signature++;
}
const Class* Class::lookup(const char* name)
// Find the class descriptor for class "name"
{
	LookupKey* asc = dictionary().assocAt(String(name));
	if (asc == 0) return 0;
	return castdown(asc->value());
}

class ReadFromTblMgr {
	ReadFromTbl& tbl;
	friend Object* Class::readFrom(OIOin& strm) const;
	friend Object* Class::readFrom(OIOifd& fd) const;
	ReadFromTblMgr(ReadFromTbl& t) : tbl(t) {
		if (tbl.level++ == 0) tbl.reset();
	}
	~ReadFromTblMgr() { --tbl.level; }
};

Object* Class::readFrom(OIOin& strm) const
{
	ReadFromTblMgr mgr(strm);
	return strm.readObject(*this);
}

Object* Class::readObject(OIOin& strm) const
// Called by OIOin::readObject()
{
#ifdef DEBUG_OBJIO
	cerr << "readFrom: read instance of class " << name();
#endif
	strm.addObjectEnable();
	const Class* saveClass = strm.readFromClass(this);
	Object* target = (*inst_reader)(strm);
	strm.readFromClass(saveClass);
	return target;
}

Object* Class::readFrom(OIOifd& fd) const
{
	ReadFromTblMgr mgr(fd);
	return fd.readObject(*this);
}

Object* Class::readObject(OIOifd& fd) const
// Called by OIOifd::readObject()
{
#ifdef DEBUG_OBJIO
	cerr << "readFrom: read instance of class " << name();
#endif
	fd.addObjectEnable();
	const Class* saveClass = fd.readFromClass(this);
	Object* target = (*inst_binreader)(fd);
	fd.readFromClass(saveClass);
	return target;
}

void Class::storer(OIOout&) const { shouldNotImplement("storer"); }

void Class::storer(OIOofd&) const { shouldNotImplement("storer"); }
