#ifndef FDSET_H
#define FDSET_H

/*$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/FDSet.h,v 3.19 92/12/19 15:39:32 sandy Exp $*/

/* FDSet.h -- Class for manipulating fd_set objects used by select(2)

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

Modification History:

$Log:	FDSet.h,v $
 * Revision 3.19  92/12/19  15:39:32  sandy
 * hpux compatible
 * 
 * Revision 3.18  92/09/21  11:18:27  sandy
 * compatible with hpux
 * 
 * Revision 3.17  92/09/11  13:57:25  sandy
 * g++ compatible
 * 
 * Revision 3.16  92/08/12  12:31:22  sandy
 * external decl for getdtablesize added for defined(sgi)
 * 
 * Revision 3.15  92/07/26  14:32:17  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.14  92/07/19  14:34:03  sandy
 * sgi compatible
 * 
 * Revision 3.13  92/06/17  11:38:57  sandy
 * __DECCXX compatible
 * 
 * Revision 3.12  92/05/28  16:29:01  sandy
 * corrected overloaded functions
 * 
 * Revision 3.11  92/05/25  11:01:11  sandy
 * __DECCXX compatible
 * 
 * Revision 3.10  91/02/18  21:06:24  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.2  91/02/07  11:29:22  kgorlen
 * Change 2nd argument to FD_ISSET to const fd_set*.
 * Cast 2nd argument to FD_ISSET to fd_set* in calls--
 * FD_ISSET is a system macro on some systems.
 * 
 * Revision 3.1  90/11/10  20:14:57  kgorlen
 * Make compare() private.
 * 
 * Revision 3.0  90/05/20  00:19:35  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Object.h>
#ifdef __DECCXX
#include <stdlib.h>     
extern "C" {
    int bcmp(const void* b1, const void* b2, int length);
    void bzero(void *b1, int length);
    int getdtablesize();
}
#else
#include <libc.h>
#endif

#include <sys/types.h>

#ifdef __GNUG__ /* actually this depends on libg++ */
#include <memory.h>
#endif

#if defined(sgi)
extern "C" int getdtablesize();
#include <memory.h>
#include <bstring.h>
#endif

#ifndef	FD_SETSIZE

#include <nihcl/nihclconfig.h>

const unsigned FD_SETSIZE = sizeof(fd_set) * 8;

inline void FD_SET(int	fd, fd_set* p)	{ ((int*)p)[div_bitsize_int(fd)] |= (1 << mod_bitsize_int(fd)); }
inline void FD_CLR(int	fd, fd_set* p)	{ ((int*)p)[div_bitsize_int(fd)] &= ~(1 << mod_bitsize_int(fd)); }
inline bool FD_ISSET(int fd, const fd_set* p)	{ return ((int*)p)[div_bitsize_int(fd)] & (1 << mod_bitsize_int(fd)); }
inline void FD_ZERO(fd_set* p)		{ memset((void*)p, 0, sizeof(fd_set)); }

#endif

#if defined(hpux)
#include <strings.h>
inline int getdtablesize() { return howmany(FD_SETSIZE,NFDBITS); }
#endif

class FDSet;

class FDSetRef: public NIHCL {
	fd_set*	p;	// pointer to word containing bit
	int fd;		// file descriptor
	FDSetRef(fd_set& fdset, int fdesc) {
		p = &fdset; fd = fdesc;
	}
	FDSetRef(const FDSetRef& s) {
		p = s.p; fd = s.fd;
	}
	friend FDSet;
public:
	operator bool() const	{ return FD_ISSET(fd,(fd_set*)p); }
	bool operator=(bool b) {
		if (b) FD_SET(fd,p);
		else FD_CLR(fd,p);
		return b;
	}
	void operator&=(bool b) {
		if (b && FD_ISSET(fd,p)) FD_SET(fd,p);
		else FD_CLR(fd,p);
	}
	void operator|=(bool b) {
		if (b || FD_ISSET(fd,p)) FD_SET(fd,p);
		else FD_CLR(fd,p);
	}
	void operator^=(bool b) {
		if ((b != 0) != (FD_ISSET(fd,p) !=0 )) FD_SET(fd,p);
		else FD_CLR(fd,p);
	}
};

class FDSet: public VIRTUAL Object {
	DECLARE_MEMBERS(FDSet);
	static unsigned	dtablesz;   // size of file descriptor table
public:
	static unsigned dtablesize() { return dtablesz; }
private:
	fd_set fs;		    // system-defined file descriptor set
protected:		// storer() functions for object I/O
	virtual void storer(OIOofd&) const;
	virtual void storer(OIOout&) const;
public:
	FDSet()				{ FD_ZERO(&fs); }
	operator fd_set&()		{ return fs; }
	operator const fd_set&() const	{ return fs; }
	operator fd_set*()		{ return &fs; }
	operator const fd_set*() const	{ return &fs; }
	FDSetRef operator[](int	fd)		{ return FDSetRef(fs,fd); }
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate operator[]() and operator[]() const
	const FDSetRef operator[](long fd) const	{ return FDSetRef(((FDSet*)this)->fs,fd); }
#else
	const FDSetRef operator[](int fd) const	{ return FDSetRef(((FDSet*)this)->fs,fd); }
#endif
	FDSet operator-(const FDSet&) const;
	FDSet operator&(const FDSet&) const;
	FDSet operator|(const FDSet&) const;
	FDSet operator^(const FDSet&) const;
#if defined(hpux)
	bool operator==(const FDSet& a) const 
        { return !bcmp((char*)&(((FDSet*)this)->fs),(char*)&a.fs,sizeof(fs)); }
#else
	bool operator==(const FDSet& a) const 
		{ return !bcmp(&(((FDSet*)this)->fs),&a.fs,sizeof(fs)); }
#endif
	bool operator!=(const FDSet& a)	const	{ return !(*this==a); }
	void operator-=(const FDSet&);
	void operator&=(const FDSet&);
	void operator|=(const FDSet&);
	void operator^=(const FDSet&);
	void clr(int fd)	{ FD_CLR(fd,&fs); }
	bool includes(unsigned fd) const { return FD_ISSET(fd,(fd_set*)&(((FDSet*)this)->fs)); }
	bool isSet(int fd) const	{ return FD_ISSET(fd,(fd_set*)&(((FDSet*)this)->fs)); }
	void set(int fd)	{ FD_SET(fd,&fs); }
	void zero()		{ FD_ZERO(&fs); }
	virtual	unsigned capacity() const;  // returns descriptor table size
	virtual	void deepenShallowCopy();   // {}
	virtual unsigned hash() const;
	virtual bool isEmpty() const;
	virtual bool isEqual(const Object&) const;
	virtual void printOn(ostream& strm =cout) const;
	virtual	unsigned size()	const;	    // returns # of fd bits set
	virtual const Class* species() const;
private:				// shouldNotImplement()
	virtual	int compare(const Object&) const;
};

#endif
