/* IdentDict.c -- implementation of Identifier Dictionary

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	October, 1985

Function:
	
An IdentDict is like a Dictionary, except keys are compared using
isSame() rather than isEqual().

$Log:	IdentDict.c,v $
 * Revision 3.14  92/12/19  15:53:06  sandy
 * removed extraneous semi-colon
 * 
 * Revision 3.13  92/07/26  14:32:24  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/04/30  15:48:21  sandy
 * one initor in DEFINE_CLASS macros
 * 
 * Revision 3.11  91/07/16  13:29:16  kgorlen
 * Modify readFrom() constructors to call _reader().
 * 
 * Revision 3.10  91/02/18  21:06:36  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.2  90/11/15  14:28:28  kgorlen
 * Include nihclIO.h
 * 
 * Revision 3.1  90/11/10  20:16:51  kgorlen
 * Eliminate operator=().
 * Implement hashOf() to fix bug in remove().
 * 
 * Revision 3.0  90/05/20  00:19:48  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/IdentDict.h>
#include <nihcl/LookupKey.h>
#include <nihcl/nihclIO.h>

#define	THIS	IdentDict
#define	BASE	Dictionary
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_CLASS(IdentDict,0,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/IdentDict.c,v 3.14 92/12/19 15:53:06 sandy Exp $",NULL)

IdentDict::IdentDict(unsigned size) : BASE(size) {}

#ifndef BUG_TOOBIG
// yacc stack overflow
IdentDict::IdentDict(const IdentDict& c) : BASE(c) {}
#endif

int IdentDict::findIndexOf(const Object& ob) const
/*
Search this IdentDict for a LookupKey with the same key object as the
argument.

Enter:
	ob = pointer to LookupKey to search for

Returns:
	index of object if found or of nil slot if not found
	
Algorithm L, Knuth Vol. 3, p. 519
*/
{
	register int i;
	Object* keyob = LookupKey::castdown(ob).key();
	for (i = h((unsigned)keyob); contents[i]!=nil; i = (i-1)&mask) {
		if (LookupKey::castdown(contents[i])->key()->isSame(*keyob)) return i;
	}
	return i;
}

unsigned IdentDict::hashOf(const Object& ob) const
{
	return (unsigned)LookupKey::castdown(ob).key();
}

Object* IdentDict::atKey(const Object& key) const
{
	return Dictionary::atKey(LookupKey((Object&)key));
}

Object* IdentDict::atKey(const Object& key, Object& newValue)
{
	return Dictionary::atKey(LookupKey((Object&)key), newValue);
}

LookupKey* IdentDict::assocAt(const Object& key) const
{
	return Dictionary::assocAt(LookupKey((Object&)key));
}

bool IdentDict::includesKey(const Object& key) const
{
	return Dictionary::includesKey(LookupKey((Object&)key));
}

IdentDict::IdentDict(OIOin& strm)
:
#ifdef MI
	Object(strm),
#endif
	BASE(strm)
{
	_reader(strm);
}

IdentDict::IdentDict(OIOifd& fd)
:
#ifdef MI
	Object(fd),
#endif
	BASE(fd)
{
	_reader(fd);
}
