/* IdentSet.c -- implementation of Identity Set

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	April, 1988

Function:
	
An IdentSet is like a Set, except keys are compared using
isSame() rather than isEqual().

$Log:	IdentSet.c,v $
 * Revision 3.14  92/12/19  15:53:07  sandy
 * removed extraneous semi-colon
 * 
 * Revision 3.13  92/07/26  14:32:27  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/04/30  15:48:22  sandy
 * one initor in DEFINE_CLASS macros
 * 
 * Revision 3.11  91/07/16  13:29:29  kgorlen
 * Modify readFrom() constructors to call _reader().
 * 
 * Revision 3.10  91/02/18  21:06:41  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.2  90/11/15  14:28:38  kgorlen
 * Include nihclIO.h
 * 
 * Revision 3.1  90/11/10  20:21:47  kgorlen
 * Implement hashOf() to fix bug in remove().
 * 
 * Revision 3.0  90/05/20  00:19:50  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/IdentSet.h>
#include <nihcl/nihclIO.h>

#define	THIS	IdentSet
#define	BASE	Set
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_CLASS(IdentSet,0,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/IdentSet.c,v 3.14 92/12/19 15:53:07 sandy Exp $",NULL)

IdentSet::IdentSet(unsigned size) : BASE(size) {}

int IdentSet::findIndexOf(const Object& ob) const
/*
Search this IdentSet for the argument object.

Enter:
	ob = object to search for

Returns:
	index of object if found or of nil slot if not found
	
Algorithm L, Knuth Vol. 3, p. 519
*/
{
	register int i;
	for (i = h((unsigned)&ob); contents[i]!=nil; i = (i-1)&mask) {
		if (contents[i]->isSame(ob)) return i;
	}
	return i;
}

unsigned IdentSet::hashOf(const Object& ob) const
{
	return (unsigned)&ob;
}

IdentSet::IdentSet(OIOin& strm)
:
#ifdef MI
	Object(strm),
#endif
	BASE(strm)
{
	_reader(strm);
}

IdentSet::IdentSet(OIOifd& fd)
:
#ifdef MI
	Object(fd),
#endif
	BASE(fd)
{
	_reader(fd);
}
