/* KeySortCltn.c  -- implementation of class KeySortCltn

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	S. M. Orlow
	Systex Inc.
	Beltsville, MD 20705

Contractor:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	February, 1987

Function:
	
Modification History:

$Log:	KeySortCltn.c,v $
 * Revision 3.13  92/12/19  15:53:09  sandy
 * removed extraneous semi-colon
 * 
 * Revision 3.12  92/07/26  14:32:33  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.11  92/04/30  15:48:26  sandy
 * one initor in DEFINE_CLASS macros
 * 
 * Revision 3.10  91/02/18  21:06:53  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.1  90/11/10  20:23:25  kgorlen
 * Fix addContentsBefore() to include *all* keys
 * equal to specified key in objects added to
 * collection.
 * Eliminate deepenShallowCopy(), occurrencesOfKey(),
 * includesKey(), removeKey(), findIndexOf(), and
 * findRangeOfKey().
 * 
 * Revision 3.0  90/05/20  00:19:58  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/nihclIO.h>
#include <nihcl/Assoc.h>
#include <nihcl/KeySortCltn.h>

#define	THIS	KeySortCltn
#define	BASE	SortedCltn
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_CLASS(KeySortCltn,1,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/KeySortCltn.c,v 3.13 92/12/19 15:53:09 sandy Exp $",NULL)

KeySortCltn::KeySortCltn(int size) : BASE(size) {}

#ifndef BUG_TOOBIG
// yacc stack overflow
KeySortCltn::KeySortCltn(const KeySortCltn& c) : BASE(c) {}
#endif

LookupKey* KeySortCltn::assocAt(int i)
{
	return LookupKey::castdown(at(i));
}

Object* KeySortCltn::keyAt(int i)
{
	LookupKey& assoc = *assocAt(i);
	return assoc.key();
}

Object* KeySortCltn::valueAt(int i)
{
	LookupKey& assoc = *assocAt(i);
	return assoc.value();
}

Object* KeySortCltn::atKey(Object& key)
{
	// binary search for key
	int i = findIndexOf(key);

	if (i > -1 && key.compare(*keyAt(i)) == 0)
		return valueAt(i);
	else 
		return nil; // key not found
}

Assoc* KeySortCltn::addAssoc(Object& key, Object& val)
{
	Assoc* assoc = new Assoc(key,val);
	SortedCltn::add(*assoc);
	return assoc;
}

Collection& KeySortCltn::addValuesTo(Collection& c) const
{
	DO(*this,LookupKey,as) c.add(*(as->value())); OD
	return c;
}

Collection& KeySortCltn::addKeysTo(Collection& c) const
{
	DO(*this,LookupKey,as) c.add(*(as->key())); OD
	return c;
}

Collection& KeySortCltn::addContentsBeforeTo(Object& key,Collection& c)
{
	if (isEmpty()) return c;
	int k = findIndexOfLastKey(findIndexOf(key));
	if (k<0) return c;
	for (int i=0; i<=k; i++ )
		c.add(*at(i));
	return c;
}

KeySortCltn::KeySortCltn(OIOin& strm)
:
#ifdef MI
	Object(strm),
#endif
	BASE(strm)
{
}

KeySortCltn::KeySortCltn(OIOifd& fd)
:
#ifdef MI
	Object(fd),
#endif
	BASE(fd)

{
}
