#ifndef OIOTBL_H
#define OIOTBL_H

/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/OIOTbl.h,v 3.12 92/07/26 14:32:57 sandy Exp $*/

/* OIOTbl.h -- Specification of tables used by storeOn()/readFrom()

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892
	July, 1988

Modification History:

$Log:	OIOTbl.h,v $
 * Revision 3.12  92/07/26  14:32:57  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.11  91/07/16  13:33:32  kgorlen
 * Implement ReadFromTbl::readFromClass() functions.
 * Change return type of ReadFromTbl::add(Object&) from
 * int to void.
 * 
 * Revision 3.10  91/02/18  21:07:27  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.3  91/02/07  11:34:54  kgorlen
 * Change argument to _storeVBase() to const void*.
 * 
 * Revision 3.2  90/11/18  17:33:48  kgorlen
 * Make StoreOnTbl::add(), addClass(), and addMember() protected.
 * Make ReadFromTbl::addObjectEnable(), add(Object&), and add(const Class*) protected.
 * 
 * Revision 3.1  90/11/13  22:16:50  kgorlen
 * Remove Object I/O related members from Class;
 * inherit from ReadFromTbl and StoreOnTbl instead.
 * 
 * Revision 3.0  90/05/20  00:20:25  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/IdentDict.h>
#include <nihcl/IdentSet.h>
#include <nihcl/OrderedCltn.h>

class StoreOnTblMgr;

class StoreOnTbl : public NIHCL {
	unsigned long level;		// storeOn() nesting level
	IdentDict d;			// object address => object number
	int objNum;			// last object number used
	OrderedCltn c;			// table of classes stored, indexed by class number
	unsigned short classNum;	// last class number used
#ifdef MI
	IdentSet storeVBaseTbl;		// table used by virtual base classes during storeOn()
#endif
	friend StoreOnTblMgr;
	void reset();
protected:
	StoreOnTbl(unsigned size =256);
	bool add(const Object&, int& num);
	bool addClass(const Class*, unsigned short& num);
	void addMember(const Object&);
public:
	~StoreOnTbl();
	unsigned long storeOnLevel()	{ return level; }
#ifdef MI
	bool _storeVBase(const void*);
#endif
	OrderedCltn classesStored();
	OrderedCltn objectsStored();
	OrderedCltn membersStored();
	virtual void dumpOn(ostream& strm =cerr) const;
};

class ReadFromTblMgr;

class ReadFromTbl : public NIHCL {
	unsigned long level;		// readFrom() nesting level
	const Class* currentClass;	// current class being read
	unsigned long addObject;	// add object to readFromTbl when 0
	OrderedCltn obs;		// table of objects for readFrom()
	OrderedCltn cls;		// table of classes for readFrom()
	friend ReadFromTblMgr;
	void reset();
protected:
	ReadFromTbl(unsigned size =1024);
	void addObjectEnable()		{ addObject = 0; }
	void add(Object&);
	int add(const Class*);
	const Class* readFromClass(const Class*);
public:
	Object* objectAt(int);
	const Class* classAt(int);
	const Class* readFromClass()	{ return currentClass; }
	unsigned long readFromLevel()	{ return level; }
	OrderedCltn classesRead();
	OrderedCltn objectsRead();
	virtual void dumpOn(ostream& strm =cerr) const;
};

#endif
