#ifndef	OIOFD_H
#define	OIOFD_H

/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/OIOfd.h,v 3.14 92/07/26 14:32:59 sandy Exp $*/

/* IOfd.h -- declarations for NIHCL file Object I/O classes

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892
	May, 1989

Modification History:

$Log:	OIOfd.h,v $
 * Revision 3.14  92/07/26  14:32:59  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.13  92/05/25  12:10:57  sandy
 * added workaround to DECCXX bug in unistd.h
 * 
 * Revision 3.12  92/05/25  11:53:15  sandy
 * *** empty log message ***
 * 
 * Revision 3.11  92/05/25  11:01:26  sandy
 * __DECCXX compatible
 * 
 * Revision 3.10  91/02/18  21:07:30  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.3  90/11/25  00:04:38  kgorlen
 * Make Class::readObject(OIOifd&) a friend of OIOifd.
 * 
 * Revision 3.2  90/11/18  17:35:46  kgorlen
 * Make Object::Object(OIOifd&) a friend of OIOifd.
 * Make Object::storeMemberOn(OIOofd&) const a friend of OIOofd.
 * 
 * Revision 3.1  90/11/13  22:17:01  kgorlen
 * Remove Object I/O related members from Class;
 * inherit from ReadFromTbl and StoreOnTbl instead.
 * 
 * Revision 3.0  90/05/20  00:20:28  kgorlen
 * Release for 1st edition.
 * 
*/
#include <nihcl/Object.h>
#include <nihcl/OIOTbl.h>
#include <string.h>
#ifdef __DECCXX
#include <unistd.h>
#include <stdlib.h>
#else
#include <osfcn.h>
#endif

class OIOifd : public ReadFromTbl {
	int fd;			// file descriptor
protected:
	virtual Object* readObject(const Class&);
	friend Object* Class::readFrom(OIOifd&) const;
	friend Object* Class::readObject(OIOifd&) const;
	friend Object::Object(OIOifd&);
public:
	OIOifd(int filedesc, unsigned size =1024) : ReadFromTbl(size)	{ fd = filedesc; }
	operator int()		{ return fd; }
	OIOifd& operator>>(char& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(unsigned char& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(short& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(unsigned short& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(int& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(unsigned int& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(long& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(unsigned long& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(float& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	OIOifd& operator>>(double& val) {
		get((char*)&val,sizeof(val));
		return *this;
	}
	int get()			{ char c; get(c); return c; }
	OIOifd&	get(char& c)		{ get(&c,1); return *this; }
	OIOifd&	get(unsigned char& c)	{ get(&c,1); return *this; }
	OIOifd& get(char*, unsigned size);
	OIOifd& get(unsigned char* val, unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(short* val, unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(unsigned short* val, unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(int* val, unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(unsigned int* val, unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(long* val,unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(unsigned long* val,unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(float* val,unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& get(double* val,unsigned size) {
		get((char*)val,size*sizeof(*val));
		return *this;
	}
	OIOifd& getCString(char*, unsigned maxlen);
};

class OIOofd : public StoreOnTbl {
	int fd;			// file descriptor
	void writeErr();
protected:
	virtual void storeObject(const Object&);
	friend void Object::storeOn(OIOofd&) const;
	friend void Object::storeMemberOn(OIOofd&) const;
public:
	OIOofd(int filedesc, unsigned size =256) : StoreOnTbl(size)	{ fd = filedesc; }
	operator int()		{ return fd; }
	OIOofd& operator<<(const char* val) {
		*this << (unsigned)strlen(val);
		put(val,strlen(val));
		return *this;
	}
	OIOofd& operator<<(char val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(unsigned char val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(short val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(unsigned short val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(int val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(unsigned int val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(long val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(unsigned long val) {
		put((char*)&val,sizeof(val));
		return *this;
	}
	OIOofd& operator<<(float val) {
		put((char*)&val,sizeof(float));
		return *this;
	}
	OIOofd& operator<<(double val) {
		put((char*)&val,sizeof(double));
		return *this;
	}
	OIOofd& put(const char* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const unsigned char* val,unsigned size) {
		return put((const char*)val, size);
	}
	OIOofd& put(const short* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const unsigned short* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const int* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const unsigned int* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const long* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const unsigned long* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const float* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& put(const double* val,unsigned size) {
		write((const char*)val,size*sizeof(*val));
		return *this;
	}
	OIOofd& putCString(const char*);
	OIOofd& write(const char* buf,unsigned nbyte) {
#ifdef __DECCXX
// Workaround DEC CXX BUG in unistd.h
// extern int	write(int fildes, char *buf, unsigned nbyte);
		if (::write(fd,(char*)buf,nbyte) < 0) writeErr();
#else
		if (::write(fd,buf,nbyte) < 0) writeErr();
#endif
		return *this;
	}
public:				// type definitions
	enum oioRecordTy {		// binary object I/O record type codes
		storeOnClassRef,	// class reference and object stored
		storeOnClass,		// class and object stored
		storeOnObjectRef	// object reference stored
	};

};

#endif /* OIOFD_H */
