/* SeqCltn.c -- implementation of abstract sequential collections

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	September, 1985

Function:
	
SeqCltn is an abstract class representing collections whose elements are
ordered and are externally named by integer indices.

$Log:	SeqCltn.c,v $
 * Revision 3.13  92/07/26  14:33:36  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/06/17  11:42:28  sandy
 * __DECCXX compatible
 * 
 * Revision 3.11  92/04/30  15:43:39  sandy
 * one initor in DEFINE_CLASS macros
 * added member SeqCltn::sameContentsInOrderAs()
 * 
 * Revision 3.10  91/02/18  21:08:15  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.2  90/11/15  14:35:55  kgorlen
 * Include nihclIO.h
 * 
 * Revision 3.1  90/11/10  21:33:35  kgorlen
 * Fix isEqual() to return NO if argument is not kindOf()
 * SeqCltn instead of raising an error.
 * 
 * Revision 3.0  90/05/20  00:21:12  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/SeqCltn.h>
#include <nihcl/nihclIO.h>

#define	THIS	SeqCltn
#define	BASE	Collection
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_ABSTRACT_CLASS(SeqCltn,0,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/SeqCltn.c,v 3.13 92/07/26 14:33:36 sandy Exp Locker: sandy $",NULL)

extern const int NIHCL_RDABSTCLASS,NIHCL_INDEXRANGE;

SeqCltn::SeqCltn() {}

SeqCltn::SeqCltn(OIOifd& fd) : BASE(fd) {}

SeqCltn::SeqCltn(OIOin& strm) : BASE(strm) {}

int SeqCltn::compare(const Object& arg) const
// Compare two sequenced collections.  If *this > arg return >0,
// *this == arg return 0, and if *this < arg return <0.
{
	assertArgClass(arg,*SeqCltn::desc(),"compare");
	Iterator i(*this);
	Iterator j(castdown(arg));
	Object*	p;	// pointer to next object in this SeqCltn
	Object*	q;	// pointer to next object in arg SeqCltn

	while ((q = j++, p = i++)) {
// previous elements compared equal; longer SeqCltn is therefore larger
		if (q == 0) return 1;
// compare() != 0 at any element determines ordering
		int val;
		if ((val = p->compare(*q)) != 0) return val;
	}
// all elements in this SeqCltn compare() equal to arg SeqCltn
	if (q == 0) return 0;	// size() == arg.size()
	return -1;
}

bool SeqCltn::sameContentsInOrderAs(const SeqCltn& c) const
{
    if ( size()==0&&c.size()==0 ) return YES;
    if ( size()!=c.size() ) return NO;
    for ( register int i=0; i< size(); i++ )
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
        if ( !at((long)i)->isSame(*c.at((long)i)) ) return NO;
#else
        if ( !at(i)->isSame(*c.at(i)) ) return NO;
#endif
    return YES;
}

#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
Object* SeqCltn::first() const { return (Object*)at((long)0); }
#else
Object* SeqCltn::first() const { return (Object*)at(0); }
#endif
	
unsigned SeqCltn::hash() const
{
	unsigned h = size();
	DO(*this,Object,p) h ^= p->hash(); OD
	return h;
}

int SeqCltn::indexOf(const Object& ob) const
{
	int i = 0;
	DO(*this,Object,p)
		if (p->isEqual(ob)) return i;
		i++;
	OD
	return -1;
}
	
bool SeqCltn::isEqual(const Object& ob) const
{
	if (!ob.isKindOf(*SeqCltn::desc())) return NO;
	if (size() != ob.size()) return NO;
	Iterator i(*this);
	Iterator j(castdown(ob));
	Object* p;
	while (p = i++) if (!p->isEqual(*(j++))) return NO;
	return YES;
}

const Class* SeqCltn::species() const	{ return &classDesc; }

Object* SeqCltn::last() const { return (Object*)at(size()-1); }

unsigned SeqCltn::occurrencesOf(const Object& ob) const
{
	unsigned n = 0;
	DO(*this,Object,p) if (p->isEqual(ob)) n++; OD
	return n;
}
	
Object* SeqCltn::doNext(Iterator& pos) const
{
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
	if (pos.index < size()) return (Object*)at((long)pos.index++);
#else
	if (pos.index < size()) return (Object*)at(pos.index++);
#endif
	return 0;
}

void SeqCltn::indexRangeErr() const
{
	setError(NIHCL_INDEXRANGE,DEFAULT,this,className());
}
