#ifndef	SET_H
#define	SET_H

/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Set.h,v 3.14 92/07/26 14:33:44 sandy Exp $*/

/* Set.h -- declarations for hash tables

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

$Log:	Set.h,v $
 * Revision 3.14  92/07/26  14:33:44  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.13  92/06/17  11:35:23  sandy
 * __DECCCXX compatible
 * 
 * Revision 3.12  92/05/28  15:26:20  sandy
 * correct at() overloading
 * 
 * Revision 3.11  92/03/18  16:51:05  sandy
 * call non-virtual Set::addToSet() in Set::reSize()-u
 * 
 * Revision 3.10  91/02/18  21:08:20  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.1  90/11/10  21:38:38  kgorlen
 * Implement hadhOf().
 * 
 * Revision 3.0  90/05/20  00:21:18  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Collection.h>
#include <nihcl/ArrayOb.h>

class Set: public Collection {
	DECLARE_MEMBERS(Set);
	unsigned count;		// number of objects in set 
	unsigned nbits;		// log base 2 of contents.capacity() 
protected:
	unsigned mask;		// contents.capacity()-1 
	ArrayOb contents;	// array of set objects 
	unsigned setCapacity(unsigned);	// compute set allocation size 
	int h(unsigned long) const;	// convert hash key into contents index 
	virtual	int findIndexOf(const Object&) const;	// search method
	virtual	unsigned hashOf(const Object&) const;	// hash method
protected:		// storer() functions for object I/O
	virtual void storer(OIOofd&) const;
	virtual void storer(OIOout&) const;
public:
	Set(unsigned size =DEFAULT_CAPACITY);
	bool operator==(const Set&) const;
	bool operator!=(const Set& a) const	{ return !(*this==a); }
	Set operator&(const Set&) const;	// intersection 
	Set operator|(const Set&) const;	// union 
	Set operator-(const Set&) const;	// difference 
        Object* addToSet(Object&);
	virtual	Object*	add(Object&);
	virtual Object*& at(int);
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
	virtual const Object *const& at(long) const;
#else
	virtual const Object *const& at(int) const;
#endif
	virtual unsigned capacity() const;
	virtual void deepenShallowCopy();
	virtual Object*	doNext(Iterator&) const;
	virtual	Object* findObjectWithKey(const Object&) const;
	virtual unsigned hash() const;
	virtual bool isEqual(const Object&) const;
	virtual unsigned occurrencesOf(const Object&) const;
	virtual void reSize(unsigned);
	virtual Object*	remove(const Object&);
	virtual void removeAll();
	virtual unsigned size() const;
	virtual const Class* species() const;
private:				// shouldNotImplement()
	virtual int compare(const Object&) const;
};

#endif
