#ifndef	STRING_H
#define	STRING_H

/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/String.h,v 3.14 92/09/11 14:03:57 sandy Exp $*/

/* String.h -- declarations for character strings

	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Authors:
	C. J. Eppich and K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892

$Log:	String.h,v $
 * Revision 3.14  92/09/11  14:03:57  sandy
 * g++ compatible
 * 
 * Revision 3.13  92/07/26  14:33:53  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/06/17  11:35:37  sandy
 * __DECCCXX compatible
 * 
 * Revision 3.11  92/05/28  15:50:22  sandy
 * corrected overloaded at()
 * 
 * Revision 3.10  91/02/18  21:08:41  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.1  91/02/18  13:45:32  kgorlen
 * Change String(const char&, unsigned l, unsigned extra) to
 * String(char, unsigned l, unsigned extra).
 * Add String(int extra) for ARM compliance.
 * 
 * Revision 3.0  90/05/20  00:21:38  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Object.h>
#include <string.h>

#ifdef __GNUG__
#include <stdlib.h>
#else
#include <malloc.h>
#endif

const unsigned DEFAULT_STRING_EXTRA = 15;
class String;
class Range;

class SubString : public NIHCL {
	char* sp;		// substring pointer 
        unsigned sl;		// substring length 
	String* st;		// String this is a SubString of
	SubString(const String&, unsigned, unsigned);
	SubString(const SubString& ss) {
		sp = ss.sp; sl = ss.sl; st = ss.st;
	}
	int compare(const char*) const;
	int compare(const String&) const;
	int compare(const SubString&) const;
	void replace(const char* st, unsigned ln);
	void checkSubStr() const;
	friend String;
public:
	const char* ptr() const	{ return sp; }
	unsigned position() const;
	unsigned length() const	{ return sl; }

	void operator=(const String&);
	void operator=(const SubString&);
	void operator=(const char*);

	bool operator<(const String&) const;
	bool operator>(const String&) const;
	bool operator<=(const String&) const;
	bool operator>=(const String&) const;
	bool operator==(const String&) const; 
	bool operator!=(const String&) const;

	bool operator<(const SubString& ss) const	{ return compare(ss)<0; } 
	bool operator>(const SubString& ss) const	{ return compare(ss)>0; }
	bool operator<=(const SubString& ss) const	{ return compare(ss)<=0; }
	bool operator>=(const SubString& ss) const	{ return compare(ss)>=0; }
	bool operator==(const SubString& ss) const	{ return compare(ss)==0; }
	bool operator!=(const SubString& ss) const	{ return compare(ss)!=0; }

	bool operator<(const char* cs) const	{ return compare(cs) < 0; }   
	bool operator>(const char* cs) const	{ return compare(cs) > 0; }   
	bool operator<=(const char* cs) const	{ return compare(cs) <= 0; }
	bool operator>=(const char* cs) const	{ return compare(cs) >= 0; }
	bool operator==(const char* cs) const	{ return compare(cs) == 0; }
	bool operator!=(const char* cs) const	{ return compare(cs) != 0; }

	friend bool operator<(const char* cs, const SubString& ss) {
	    return ss.compare(cs) > 0;
	}
	friend bool operator>(const char* cs, const SubString& ss) {	 
	    return ss.compare(cs) < 0;
	}
	friend bool operator<=(const char* cs, const SubString& ss) { 
	    return ss.compare(cs) >= 0; 
	}
	friend bool operator>=(const char* cs, const SubString& ss) { 
	    return ss.compare(cs) <= 0;
	}
	friend bool operator==(const char* cs, const SubString& ss) {
	    return ss.compare(cs) == 0; 
	}
	friend bool operator!=(const char* cs, const SubString& ss) {
	    return ss.compare(cs) != 0; 
	}

	String operator&(const String&) const;
	String operator&(const SubString&) const;
	String operator&(const char*) const;
	friend String operator&(const char*, const SubString&);

	void dumpOn(ostream& strm =cerr) const;
	void printOn(ostream& strm =cout) const;
};

class String: public VIRTUAL Object {
	DECLARE_MEMBERS(String);
	char* p;		// character string
	unsigned len;		// length of string, excluding null character
	unsigned alloc;		// amount of storage allocated
	void init(unsigned l =0, unsigned extra =DEFAULT_STRING_EXTRA);
	void indexRangeErr() const;
	friend SubString;
protected:		// storer() functions for object I/O
	virtual void storer(OIOofd&) const;
	virtual void storer(OIOout&) const;		
public:
	String(char c, unsigned l=1, unsigned extra=DEFAULT_STRING_EXTRA);
	String()		{ init(0); }
	String(int extra)	{ init(0,extra); }
	String(unsigned extra)	{ init(0,extra); }
	String(const char*);
	String(const char*, unsigned extra);
	String(const String&);
	String(const String&, unsigned extra);
	String(const SubString&);
	String(const SubString&, unsigned extra);
	~String();
	operator const char*() const	{ return p; }
	SubString operator()(unsigned pos, unsigned lgt);
	const SubString operator()(unsigned pos, unsigned lgt) const;
	SubString operator()(const Range& r);
	const SubString operator()(const Range& r) const;
	char& operator[](int i) {
		if (i >= len) indexRangeErr();
		return p[i];
    	}
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate operator[]() and operator[]() const
	char operator[](long i) const {
#else
	char operator[](int i) const {
#endif
		if (i >= len) indexRangeErr();
		return p[i];
    	}

	void operator=(const String&);
	void operator=(const SubString&);
	void operator=(const char*);

	bool operator<(const String& s) const	{ return strcmp(p, s.p) < 0; }
	bool operator>(const String& s) const	{ return strcmp(p, s.p) > 0; }
	bool operator<=(const String& s) const	{ return strcmp(p, s.p) <= 0; }
	bool operator>=(const String& s) const	{ return strcmp(p, s.p) >= 0; }
	bool operator==(const String& s) const;
	bool operator!=(const String& s) const	{ return !(*this==s); }

	bool operator<(const SubString& ss) const 	{ return ss.compare(*this) > 0; }
	bool operator>(const SubString& ss) const	{ return ss.compare(*this) < 0; }
	bool operator<=(const SubString& ss) const	{ return ss.compare(*this) >= 0; }
	bool operator>=(const SubString& ss) const	{ return ss.compare(*this) <= 0; }
	bool operator==(const SubString& ss) const	{ return ss.compare(*this) == 0; }
	bool operator!=(const SubString& ss) const	{ return ss.compare(*this) != 0; }

	bool operator<(const char* cs) const	{ return strcmp(p,cs) < 0; }   
	bool operator>(const char* cs) const	{ return strcmp(p,cs) > 0; }   
	bool operator<=(const char* cs) const	{ return strcmp(p,cs) <= 0; }
	bool operator>=(const char* cs) const	{ return strcmp(p,cs) >= 0; }
	bool operator==(const char* cs) const	{ return strcmp(p,cs) == 0; }
	bool operator!=(const char* cs) const	{ return strcmp(p,cs) != 0; }

	friend bool operator<(const char* cs, const String& s) {
	    return strcmp(cs, s.p) < 0;
	}
	friend bool operator>(const char* cs, const String& s) {	 
	    return strcmp(cs, s.p) > 0;
	}
	friend bool operator<=(const char* cs, const String& s) { 
	    return strcmp(cs, s.p) <= 0; 
	}
	friend bool operator>=(const char* cs, const String& s) { 
	    return strcmp(cs, s.p) >= 0;
	}
	friend bool operator==(const char* cs, const String& s) {
	    return strcmp(cs, s.p) == 0; 
	}
	friend bool operator!=(const char* cs, const String& s) {
	    return strcmp(cs, s.p) != 0; 
	}

	String operator&(const String& s) const;
	String operator&(const SubString& ss) const;
	String operator&(const char* cs) const;
	friend String operator&(const char*, const SubString&);
	friend String operator&(const char* cs, const String& s);

	String& operator&=(const String&);
	String& operator&=(const SubString&);
	String& operator&=(const char* cs);

	char& at(int i)			{ return (*this)[i]; }
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
	char at(long i) const		{ return (*this)[i]; }
#else
	char at(int i) const		{ return (*this)[i]; }
#endif
	unsigned length() const	{ return len; } 
	unsigned reSize(unsigned new_capacity);

	virtual void toAscii();
	virtual void toLower();
	virtual void toUpper();
	virtual unsigned capacity() const;
	virtual int compare(const Object& ob) const;
	virtual void deepenShallowCopy();
	virtual unsigned hash() const;
	virtual bool isEqual(const Object&) const;
	virtual void printOn(ostream& strm =cout) const;
	virtual void scanFrom(istream& strm);
	virtual unsigned size() const;
	virtual const Class* species() const;
};

inline ostream& operator<<(ostream& strm, const SubString& ss)
{
	ss.printOn(strm);
	return(strm);
}

inline SubString::SubString(const String& s, unsigned pos, unsigned lgt)
{ 
	sp = &((String&)s).p[pos];
	sl = lgt;
	st = &(String&)s;
	checkSubStr();
}
	
inline unsigned SubString::position() const	{ return sp - st->p; }

inline SubString String::operator()(unsigned pos, unsigned lgt)
{	
	return SubString(*this, pos, lgt);
}

inline const SubString String::operator()(unsigned pos, unsigned lgt) const
{	
	return SubString(*this, pos, lgt);
}

inline bool SubString::operator<(const String& s) const { return compare(s) < 0; }
inline bool SubString::operator>(const String& s) const { return compare(s) > 0; }
inline bool SubString::operator<=(const String& s) const { return compare(s) <= 0; }
inline bool SubString::operator>=(const String& s) const { return compare(s) >= 0; }
inline bool SubString::operator==(const String& s) const { return compare(s) == 0; }
inline bool SubString::operator!=(const String& s) const { return compare(s) != 0; }

#endif

