// ex11-1.c -- Scheduling lightweight processes in
//             the NIH class library

// $Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/procex/RCS/ex11-1.c,v 3.11 92/11/08 17:03:29 sandy Exp $

#define BASE StackProc
#if BASE == StackProc
#include <nihcl/StackProc.h>
#endif
#if BASE == HeapProc
#include <nihcl/HeapProc.h>
#endif

#include <nihcl/Scheduler.h>
#include <nihcl/Semaphore.h>
#include <nihcl/String.h>

class TestProcess : public BASE {
public:
    TestProcess(const char* name, stackTy* bot, int pri,
                Process* parent);
    static TestProcess* create(const char* name, int pri,
                               Process* parent);
};

TestProcess::TestProcess(const char* pname,stackTy* bot,int pri,
                         Process* parent)
            : BASE(pname,bot,pri)
{
    // parent process yields to allow this process to start
    if ( FORK() ) { Scheduler::yield(); return; }

    cout << name() << pri << " start" << endl;

    // yield to parent process
    if (parent) {
        parent->resume();
        Scheduler::yield();
        }
    // suspend until child yields
    suspend();
    Scheduler::schedule();

    cout << name() << pri << " resume" << endl;

    // terminate to avoid return
    terminate();
}

TestProcess* TestProcess::create(const char* name, int pri,
                                 Process* parent)
{
    // the next two statements must be in the same scope
    // for the address of the stack bottom to be correct
#ifdef NO_NESTED_TYPES
    auto stackTy bottom;
#else
    auto Process::stackTy bottom;
#endif
    return new TestProcess(name,&bottom,pri,parent);
}

main()
{
    // start Scheduler
    // create main context with priority 0
    MAIN_PROCESS(0);

    Process* parent =0;
    String* pname = new String("P"); 
    for (register int i=MAXPRIORITY; i>=1; i--)
        parent = TestProcess::create(*pname,i,parent);

    cout << "main Process" << endl;
    parent->resume();
    Scheduler::yield();
}
