// ex11-3.c -- Protecting a critical section of code
//             with an AutoSignal

// $Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/procex/RCS/ex11-3.c,v 3.11 92/11/08 17:03:31 sandy Exp $

#define BASE StackProc
#if BASE == StackProc
#include <nihcl/StackProc.h>
#endif
#if BASE == HeapProc
#include <nihcl/HeapProc.h>
#endif

#include <nihcl/Scheduler.h>
#include <nihcl/Semaphore.h>
#include <nihcl/String.h>

static Semaphore protect_critical(1);

class AutoSignal {
    Semaphore* as;
public:
    AutoSignal(Semaphore& s) { as= &s; as->wait(); }
    ~AutoSignal() { as->signal(); }
};

class TestProcess : public BASE {
public:
    TestProcess(const char* name, stackTy* bot, int pri);
    static TestProcess* create(const char* name, int pri);
};

TestProcess::TestProcess(const char* pname,stackTy* bot,int pri)
            : BASE(pname,bot,pri)
{
    // parent process yields to allow this process to start
    if ( FORK() ) { Scheduler::yield(); return; }

    while (1) {
        // this block represents a critical section
        // although it does nothing critical
        AutoSignal autosig(protect_critical);
        cout << name() << pri << " enter critical section" << endl;
        break;
        }

    // terminate to avoid return
    terminate();
}

TestProcess* TestProcess::create(const char* name, int pri)
{
    // the next two statements must be in the same scope
    // for the address of the stack bottom to be correct
#ifdef NO_NESTED_TYPES
    auto stackTy bottom;
#else
    auto Process::stackTy bottom;
#endif
    return new TestProcess(name,&bottom,pri);
}

main()
{
    // start Scheduler
    // create main context with priority 0
    MAIN_PROCESS(0);

    String* pname = new String("P"); 
    for (register int i=MAXPRIORITY; i>=1; i--)
        TestProcess::create(*pname,i);

    cout << "main Process" << endl;
}
